--
-- baleConverter
--
--
-- author:    	Xentro (www.fs-uk.com)(Marcus@Xentro.se)
-- @version:    v1.52
-- @date:       2014-04-02
-- 

baleConverter = {};
baleConverter.currentModDirectory = g_currentModDirectory;

local baleConverter_mt = Class(baleConverter, Object);
InitObjectClass(baleConverter, "baleConverter");

function baleConverter.onCreate(id)
    local instance = baleConverter:new(g_server ~= nil, g_client ~= nil);
	if instance:load(id) then
		g_currentMission:addOnCreateLoadedObject(instance);
		instance:register(true);
	else
		instance:delete();
	end;
end;

g_onCreateUtil.addOnCreateFunction("baleConverter", baleConverter.onCreate);

function baleConverter:new(isServer, isClient)
    local self = Object:new(isServer, isClient, baleConverter_mt);
    self.className = "baleConverter";
    
	self.baleTable = {};
	
    return self;
end;

function baleConverter:load(id)
	self.triggerId = id;
	addTrigger(id, "triggerCallback", self);
	self.isEnabled = true;
	
	self.test = Utils.getNoNil(getUserAttribute(id, "terrainHelp"), false);
	
	local husbandryName = getUserAttribute(id, "husbandryTypeName");
	if husbandryName ~= nil and husbandryName ~= "" then
		self.husbandryName = husbandryName;
	end;
	
	self.terrainThreshold = Utils.getNoNil(getUserAttribute(id, "terrainThreshold"), 0.2);
	self.baleTime = Utils.getNoNil(getUserAttribute(id, "baleTime"), 9000);
	self.deleteTime = Utils.getNoNil(getUserAttribute(id, "deleteTime"), 0);
	self.particleTime = Utils.getNoNil(getUserAttribute(id, "particleTime"), 300);
	
	local outputType = getUserAttribute(id, "outputType");
	self.outputType = Fillable.fillTypeNameToInt[outputType];
	
	if self.outputType == nil then
		if self.husbandryName ~= nil then
			self.outputType = Fillable.FILLTYPE_WHEAT_WINDROW;
		else
			print("baleConverter - Error: Invalid outputType for trigger");
			return false;
		end;
	end;
	
	local fillTypes = getUserAttribute(id, "fillTypes");
	if fillTypes ~= nil and fillTypes ~= "" then
		self.acceptedFillTypes = {};
		local types = Utils.splitString(" ", fillTypes);
		
		for k,v in pairs(types) do
			local desc = Fillable.fillTypeNameToDesc[v];
			
			if desc ~= nil then
				self.acceptedFillTypes[desc.index] = true;
			end;
		end;
		
		self.allowAllFillTypes = false;
	else
		self.allowAllFillTypes = true;
	end;
	
	local particlePositionStr = getUserAttribute(id, "particlePosition");
	if particlePositionStr ~= nil then
		local x,y,z = Utils.getVectorFromString(particlePositionStr);
		if x ~= nil and y ~= nil and z ~= nil then
			self.particlePosition = {x, y, z};
		end;
	end;
	
	local particleRotationStr = getUserAttribute(id, "particleRotation");
	if particleRotationStr ~= nil then
		self.particleRotation = Utils.getRadiansFromString(particleRotationStr, 3)
		
		if self.particleRotation == nil then
			self.particleRotation = {0, 0, 0};
		end;
	end;
	
	local particleSystem = getUserAttribute(id, "particleSystem");
	if particleSystem ~= nil then
		self.particleSystemRoot = loadI3DFile(Utils.getFilename("particleSystems/" .. particleSystem ..".i3d", baleConverter.currentModDirectory));
		
		local x, y, z = getTranslation(id);
		if self.particlePosition ~= nil then
			x = x + self.particlePosition[1];
			y = y + self.particlePosition[2];
			z = z + self.particlePosition[3];
		end;
		
		setTranslation(self.particleSystemRoot, x, y, z);
		
		setRotation(self.particleSystemRoot, unpack(self.particleRotation));
		
		link(getParent(id), self.particleSystemRoot);

		for i = 0, getNumOfChildren(self.particleSystemRoot) - 1 do
			local child = getChildAt(self.particleSystemRoot, i);
			if getClassName(child) == "Shape" then
				local geometry = getGeometry(child);
				if geometry ~= 0 then
					if getClassName(geometry) == "ParticleSystem" then
						self.particleSystem = geometry;
					end;
				end;
			end;
		end;

		if self.particleSystem ~= nil then
			setEmittingState(self.particleSystem, false);
		end;
		
		self.particlesActive = false;
	end;
	
	g_currentMission:addOnCreateLoadedObjectToSave(self)
	
	return true;
end;

function baleConverter:delete()
	g_currentMission:removeOnCreateLoadedObjectToSave(self)
	
	if self.triggerId ~= nil then
		removeTrigger(self.triggerId);
	end;
	
	if self.particleSystemRoot ~= nil then
		delete(self.particleSystemRoot);
	end;
end;

function baleConverter:loadFromAttributesAndNodes(xmlFile, tag)
	local i = 0;
	while true do
		local key = tag .. string.format(".bale(%d)", i);
		if not hasXMLProperty(xmlFile, key) then break; end;
		
		local id = getXMLInt(xmlFile, key .. "#id");
		local time = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#time"), 0);
		local deleteTime = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#deleteTime"), 0);
		local pTime = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#pTime"), 0);
		local deleted = Utils.getNoNil(getXMLBool(xmlFile, key .. "#deleted"), true);
		
		if id == nil or time == nil then
			break;
		end;
		
		local entry = {};
		entry.id = id;
		entry.time = time;
		entry.deleteTime = deleteTime;
		entry.pTime = pTime;
		entry.deleted = deleted;
		
		self.baleTable[id] = entry;
		i = i + 1;
	end;
	
	return true;
end;

function baleConverter:getSaveAttributesAndNodes(nodeIdent)
	local attributes = "";
	local nodes = "";
	
	for _, bale in pairs(self.baleTable) do
		if nodes ~= "" then
			nodes = nodes .. "\n";
		end;
		
		nodes = nodes .. nodeIdent .. '<bale';
		nodes = nodes .. ' id="' .. bale.id .. '"';
		nodes = nodes .. ' time="' .. bale.time .. '"';
		nodes = nodes .. ' deleteTime="' .. bale.deleteTime .. '"';
		nodes = nodes .. ' pTime="' .. bale.pTime .. '"';
		nodes = nodes .. ' deleted="' .. tostring(bale.deleted) .. '"';
		nodes = nodes .. ' />';
	end;
	
	return attributes, nodes;
end;

function baleConverter:update(dt)
	for k, v in pairs(self.baleTable) do
		if v ~= nil and v.id ~= nil and v.time ~= nil then
			
			if v.object ~= nil and v.object.nodeId ~= nil and v.object.nodeId > 0 then 
				local x, y, z = getWorldTranslation(v.object.nodeId);
				
				local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
				if v.test then
					print("bale "..k);
					print("terrainHeight: "..tostring(terrainHeight + self.terrainThreshold));
					print("bale height: "..tostring(y));
					if y > terrainHeight + self.terrainThreshold then
						print("add "..tostring(((terrainHeight + self.terrainThreshold) - y) + 0.05).." too terrainThreshold");
						print("too get the best result then push the bale into trigger.");
					else
						print("your terrain height is bigger then the bale and that's good.");
					end;
					v.test = false;
				end;
				
				if y ~= nil and terrainHeight + self.terrainThreshold >= y then
					v.deleteTime = v.deleteTime - dt;
					
					if v.deleteTime <= 0 then
						if self.isServer then
							if self.particleSystemRoot ~= nil then
								setTranslation(self.particleSystemRoot, x, terrainHeight, z);
							end;
							
							v.object:delete();
						end;
						v.deleted = true;
					end;
				end;
			end;
			
			if v.deleted then
				v.time = v.time - dt;
				
				if self.particleSystem ~= nil then
					if v.pTime > 0 then
						v.pTime = v.pTime - dt;
						
						if not self.particlesActive then
							setEmittingState(self.particleSystem, true);
							self.particlesActive = true;
						end;
					else
						if self.particlesActive then
							setEmittingState(self.particleSystem, false);
							self.particlesActive = false;
						end;
					end;
				end;
				
				if v.time <= 0 then
					if self.husbandryName ~= nil then
						local husbandry = g_currentMission.husbandries[self.husbandryName];
						
						if husbandry ~= nil then
							husbandry:changeFillLevels(v.fillLevel, self.outputType);
							husbandry:raiseDirtyFlags(husbandry.husbandryDirtyFlag)
							husbandry:updateStrawPlane()
						else
							print("baleConverter - Warning: husbandry type does not exist! " .. tostring(self.husbandryName));
							self.isEnabled = false;
						end;
					else
						local silo = g_currentMission:getSiloAmount(self.outputType);
						g_currentMission:setSiloAmount(self.outputType, silo + v.fillLevel)
					end;
					
					if self.isServer then
						self.baleTable[v.id] = nil;
					end;
				end;
			end;
		end;
	end;
end;

function baleConverter:updateTick(dt)
end;

function baleConverter:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
    if self.isEnabled then
		if onEnter then
			local object = g_currentMission:getNodeObject(otherId);
			if object ~= nil and object:isa(Bale) and object.getFillType ~= nil and self:allowFillType(object:getFillType()) then				
				if self.baleTable[otherId] == nil then
					local entry = {};
					entry.id = otherId;
					entry.object = object;
					entry.time = self.baleTime;
					entry.pTime = self.particleTime;
					entry.deleteTime = self.deleteTime;
					entry.deleted = false;
					entry.test = self.test;
					entry.fillLevel = object:getFillLevel();
					self.baleTable[otherId] = entry;
				end;
			end;
			
		elseif onLeave then
			if self.baleTable[otherId] ~= nil then
				self.baleTable[otherId] = nil;
			end;
		end;
	end;
end;

function baleConverter:allowFillType(fillType)
	if self.acceptedFillTypes ~= nil then
		if self.acceptedFillTypes[fillType] ~= nil then
			if self.acceptedFillTypes[fillType] then
				return true;
			end;
		end;
	end;
	
	return self.allowAllFillTypes;
end;